/*
        Universidad de Valladolid - Escuela de Ingenierias Industriales
            Grado en Electrnica Indutrial y Automtica
            Trabajo de fin de grado
            Alumno: Oscar Emilio Aponte Rengifo
            Tutor: Jess Mara Zamarreo Cosme

        Desarrollo de un Servidor OPC UA para una tarjeta de adquisicin de datos USB-1408FS-Plus

        server.c

*/

#include <signal.h>

#include "open62541.h"
#include "tarjeta_1408FS.h"

UA_Boolean running = true;
void Comun_mode(UA_Server *server);
static void
define_tarjeta (UA_Server *server, char* tarjeta){

    UA_NodeId Tarjeta_Id = UA_NODEID_STRING(1,tarjeta);
    UA_ObjectAttributes trj_Attr = UA_ObjectAttributes_default;
    trj_Attr.displayName = UA_LOCALIZEDTEXT("es-ES", tarjeta);
    UA_Server_addObjectNode(server, Tarjeta_Id,UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),UA_QUALIFIEDNAME(1, tarjeta), UA_NODEID_NUMERIC(0, UA_NS0ID_BASEOBJECTTYPE),trj_Attr, NULL, &Tarjeta_Id);
    /************************************************* Tarjeta en uso */
    UA_VariableAttributes tarjeta_serie_attr = UA_VariableAttributes_default;
    UA_Int32 tarjeta_id_num = get_tarjeta_en_uso();
    UA_Variant_setScalar(&tarjeta_serie_attr.value, &tarjeta_id_num, &UA_TYPES[UA_TYPES_INT32]);
    tarjeta_serie_attr.description = UA_LOCALIZEDTEXT("es-ES","Tarjeta en uso");
    tarjeta_serie_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Tarjeta en uso");
    tarjeta_serie_attr.dataType = UA_TYPES[UA_TYPES_INT32].typeId;
    tarjeta_serie_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    UA_NodeId serie_Id= UA_NODEID_STRING ( 1 , "Tarjeta en uso" );
    UA_Server_addVariableNode(server, serie_Id ,  Tarjeta_Id,
                              UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT),
                               UA_QUALIFIEDNAME(1, "Tarjeta en uso"),
                              UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE),tarjeta_serie_attr, NULL, NULL);
}
/******************************************* Configuracin: comn/diferencial */
static void
add_config_modo(UA_Server *server,char *tarjeta){

    UA_VariableAttributes config_attr = UA_VariableAttributes_default;
    UA_UInt16 config_id_num = configuracion_modo(); // Modo diferencial
    UA_Variant_setScalar(&config_attr.value, &config_id_num, &UA_TYPES[UA_TYPES_UINT16]);
    config_attr.description = UA_LOCALIZEDTEXT("es-ES","Modo: Diferencial/Comun");
    config_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Modo: Diferencial/Comun");
    config_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    config_attr.accessLevel = UA_ACCESSLEVELMASK_READ;
    config_attr.valueRank = -1;
    UA_NodeId config_Id= UA_NODEID_STRING ( 1 ,"Config_modo");
    UA_QualifiedName config_modo_Name = UA_QUALIFIEDNAME ( 1 , "Config_modo");
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Tarjeta_Id = UA_NODEID_STRING(1, tarjeta );
    UA_Server_addVariableNode ( server , config_Id , Tarjeta_Id ,
                              parentReferenceNodeId , config_modo_Name ,
                              variableTypeNodeId , config_attr , NULL , NULL );
}
int
read_config_modo(UA_Server *server)
{
    UA_Variant modo1;
    UA_NodeId config_Id= UA_NODEID_STRING ( 1 ,"Config_modo");
    UA_Server_readValue(server,config_Id,&modo1);
    void *modo2 = modo1.data;
    int modo3 = *(UA_UInt16 *)modo2;
    return modo3;
}
/******************************************** Define Object: Entradas Analgicas */
static void
Define_object_entradas_Analogas(UA_Server *server, char * id_Analog_in) {

    UA_NodeId Entradas_Analogas_Id = UA_NODEID_STRING(1,id_Analog_in);
    UA_ObjectAttributes entradas_Analogas_Attr = UA_ObjectAttributes_default;
    entradas_Analogas_Attr.displayName = UA_LOCALIZEDTEXT("es-ES", id_Analog_in);
    UA_Server_addObjectNode(server, Entradas_Analogas_Id,
                            UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),
                            UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
                            UA_QUALIFIEDNAME(1, id_Analog_in), UA_NODEID_NUMERIC(0, UA_NS0ID_BASEOBJECTTYPE),
                            entradas_Analogas_Attr, NULL, &Entradas_Analogas_Id);
}

/*********************************************** Entradas Analgicas - In_Volts */
static void
add_In_Volts_Variable ( UA_Server * server,char * id_Analog_in ,char* id_volts) {

    UA_Float now = 0 ;
    UA_VariableAttributes In_volts_attr = UA_VariableAttributes_default ;
    In_volts_attr. displayName = UA_LOCALIZEDTEXT ( "es-ES" , "In_Volts" );
    In_volts_attr. accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE ;
    In_volts_attr.dataType = UA_TYPES[UA_TYPES_FLOAT].typeId;
    UA_Variant_setScalar ( &In_volts_attr . value , & now , & UA_TYPES [ UA_TYPES_FLOAT ]);
    UA_NodeId In_volts_NodeId = UA_NODEID_STRING ( 1 , id_volts );
    UA_QualifiedName In_volts_Name = UA_QUALIFIEDNAME ( 1 , id_volts);
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Entradas_Analogas_Id = UA_NODEID_STRING(1, id_Analog_in );
    UA_Server_addVariableNode ( server , In_volts_NodeId , Entradas_Analogas_Id ,
                              parentReferenceNodeId , In_volts_Name ,
                              variableTypeNodeId , In_volts_attr , NULL , NULL );

}
 void
read_in_volts(UA_Server *server,char *id_nodo,int canal)
{
    UA_NodeId In_Volts_NodeId = UA_NODEID_STRING ( 1 , id_nodo);
    UA_Float now = get_In_volts(canal);
    UA_Variant In_Volts_value;
    UA_Variant_setScalar ( &In_Volts_value , & now , & UA_TYPES [ UA_TYPES_FLOAT]);
    UA_Server_writeValue ( server , In_Volts_NodeId , In_Volts_value );
}
static void
beforeRead_In_Volts_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    read_in_volts(server, "In volts CH 0",0);
}
static void
beforeRead_In_Volts_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_volts(server,"In volts CH 1",1);
}
static void
beforeRead_In_Volts_CH_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
read_in_volts(server,"In volts CH 2",2);
}
static void
beforeRead_In_Volts_CH_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
 read_in_volts(server,"In volts CH 3",3);
}
static void
beforeRead_In_Volts_CH_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_volts(server,"In volts CH 4",4);
}
static void
beforeRead_In_Volts_CH_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_volts(server,"In volts CH 5",5);
}
static void
beforeRead_In_Volts_CH_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_volts(server,"In volts CH 6",6);
}
static void
beforeRead_In_Volts_CH_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
        read_in_volts(server,"In volts CH 7",7);
}
///********************************** Entradas Analgicas - In_Volts: callback **************************************/
static void
addValueCallbackTo_in_volts(UA_Server *server, char *id_node,int canal)
{
    UA_NodeId In_volts_NodeId = UA_NODEID_STRING ( 1 , id_node );
    UA_ValueCallback callback_In_Volts_CH_0 ;
    if (canal == 0 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_0;
    if (canal == 1 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_1;
    if (canal == 2 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_2;
    if (canal == 3 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_3;
    if (canal == 4 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_4;
    if (canal == 5 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_5;
    if (canal == 6 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_6;
    if (canal == 7 )
    callback_In_Volts_CH_0 . onRead = beforeRead_In_Volts_CH_7;

    UA_Server_setVariableNode_valueCallback ( server , In_volts_NodeId , callback_In_Volts_CH_0 );
}

///********************************* Entradas Analgicas - In_Bin ***************************************************/
static void
add_In_Bin_Variable ( UA_Server * server,char * id_Analog_in ,char* id_bin) {

    UA_UInt16 now = 0 ;
    UA_VariableAttributes In_Bin_attr = UA_VariableAttributes_default ;
    In_Bin_attr . displayName = UA_LOCALIZEDTEXT ( "es-ES" , "In_Bin" );
    In_Bin_attr. accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE ;
    In_Bin_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    UA_Variant_setScalar ( &In_Bin_attr. value , & now , & UA_TYPES [ UA_TYPES_UINT16]);
    UA_NodeId In_Bin_NodeId = UA_NODEID_STRING ( 1 , id_bin );
    UA_QualifiedName In_Bin_Name = UA_QUALIFIEDNAME ( 1 , id_bin );
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Entradas_Analogas_Id = UA_NODEID_STRING(1, id_Analog_in );
    UA_Server_addVariableNode ( server , In_Bin_NodeId , Entradas_Analogas_Id ,
                              parentReferenceNodeId , In_Bin_Name ,
                              variableTypeNodeId , In_Bin_attr , NULL , NULL );
}
void read_in_bin(UA_Server *server,char *id_nodo,int canal)
{
    UA_UInt16 now = get_In_bin(canal);
    UA_Variant In_Bin_value ;
    UA_Variant_setScalar ( &In_Bin_value , & now , & UA_TYPES [ UA_TYPES_UINT16]);
    UA_NodeId In_Bin_NodeId = UA_NODEID_STRING ( 1 ,id_nodo );
    UA_Server_writeValue ( server , In_Bin_NodeId , In_Bin_value );
}
 static void
beforeRead_In_Bin_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_bin(server,"In bin CH 0",0);
}
 static void
beforeRead_In_Bin_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
        read_in_bin(server,"In bin CH 1",1);
}
 static void
beforeRead_In_Bin_CH_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_bin(server,"In bin CH 2",2);
}
 static void
beforeRead_In_Bin_CH_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
    read_in_bin(server,"In bin CH 3",3);
}
 static void
beforeRead_In_Bin_CH_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
       read_in_bin(server,"In bin CH 4",4);
}
 static void
beforeRead_In_Bin_CH_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
       read_in_bin(server,"In bin CH 5",5);
}
 static void
beforeRead_In_Bin_CH_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
   read_in_bin(server,"In bin CH 6",6);
}
 static void
beforeRead_In_Bin_CH_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data ) {
   read_in_bin(server,"In bin CH 7",7);
}
///********************************** Entradas Analgicas - In_Bin: callback ***************************************************/
static void
addValueCallbackTo_in_bin(UA_Server *server,char *id_nodo,int canal) {

    UA_NodeId In_Bin_NodeId = UA_NODEID_STRING ( 1 , id_nodo );
    UA_ValueCallback callback ;

    if (canal == 0 )
    callback . onRead = beforeRead_In_Bin_CH_0;
    if (canal == 1 )
    callback . onRead = beforeRead_In_Bin_CH_1;
    if (canal == 2 )
    callback . onRead = beforeRead_In_Bin_CH_2;
    if (canal == 3 )
    callback . onRead = beforeRead_In_Bin_CH_3;
    if (canal == 4 )
    callback . onRead = beforeRead_In_Bin_CH_4;
    if (canal == 5 )
    callback . onRead = beforeRead_In_Bin_CH_5;
    if (canal == 6 )
    callback . onRead = beforeRead_In_Bin_CH_6;
    if (canal == 7 )
    callback . onRead = beforeRead_In_Bin_CH_7;

    UA_Server_setVariableNode_valueCallback ( server , In_Bin_NodeId , callback );
}
///********************************** Entradas Analgicas - In_Rango *************************************/
static void
add_In_Rango_Variable ( UA_Server * server,char * id_Analog_in ,char* id_rango) {

    UA_NodeId Entradas_Analogas_Id = UA_NODEID_STRING(1, id_Analog_in );
    UA_Float now = 5;
    /* (+-) 20 10 5 4 2.5 2 1.25 1v */
    /* BIP20VOLTS BIP10VOLTS  BIP5VOLTS BIP4VOLTS BIP2PT5VOLTS BIP2VOLTS BIP1PT25VOLTS BIP1VOLTS*/
    UA_VariableAttributes In_Rango_attr = UA_VariableAttributes_default ;
    In_Rango_attr . displayName = UA_LOCALIZEDTEXT ( "es-ES" , "In_Rango");
    In_Rango_attr. accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE ;
    In_Rango_attr.dataType = UA_TYPES[UA_TYPES_FLOAT].typeId;
    In_Rango_attr.description = UA_LOCALIZEDTEXT("es-ES","(+-) 20 10 5 4 2.5 2 1.25 1v");
    UA_Variant_setScalar(&In_Rango_attr. value , &now, & UA_TYPES [ UA_TYPES_FLOAT ]);
    UA_NodeId In_Rango_NodeId = UA_NODEID_STRING ( 1 , id_rango );
    UA_QualifiedName In_Rango_Name = UA_QUALIFIEDNAME ( 1 , id_rango );
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_Server_addVariableNode ( server , In_Rango_NodeId , Entradas_Analogas_Id ,parentReferenceNodeId , In_Rango_Name ,variableTypeNodeId , In_Rango_attr , NULL , NULL );
}
void after_write_in_rango(UA_Server *server, char *id_node , int canal)
{
    UA_Variant In_rango_value;
    UA_NodeId range_nodeId = UA_NODEID_STRING ( 1 ,id_node);
    UA_Server_readValue ( server , range_nodeId , &In_rango_value );
    void *in_r = In_rango_value.data;
    float in_r_d = *(UA_Float *)in_r;
    bool conf = selecciona_Range(canal,in_r_d);
    if (conf == false)
    {
        float r=5;
        UA_Variant_setScalar ( &In_rango_value , &r , & UA_TYPES [ UA_TYPES_FLOAT]);
        UA_Server_writeValue(server,range_nodeId,In_rango_value);
    }
}
static void
afterWrite_In_rango_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 0",0);
}
static void
afterWrite_In_rango_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 1",1);
}
static void
afterWrite_In_rango_CH_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 2",2);
}
static void
afterWrite_In_rango_CH_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 3",3);
}
static void
afterWrite_In_rango_CH_4 (UA_Server* server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 4",4);
}
static void
afterWrite_In_rango_CH_5 (UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 5",5);
}
static void
afterWrite_In_rango_CH_6 (UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    after_write_in_rango(server,"In rango CH 6",6);
}
static void
afterWrite_In_rango_CH_7 (UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
     after_write_in_rango(server,"In rango CH 7",7);
}
/********************************** Entradas Analgicas - In_Rango: callback ***************************/
static void
addValueCallbackTo_in_rango(UA_Server *server,char *id_nodo,int canal) {

    UA_NodeId in_rango_NodeId = UA_NODEID_STRING ( 1 , id_nodo );
    UA_ValueCallback callback_rango ;

    if (canal == 0 )
    callback_rango . onWrite = afterWrite_In_rango_CH_0;
    if (canal == 1 )
    callback_rango . onWrite = afterWrite_In_rango_CH_1;
    if (canal == 2 )
    callback_rango . onWrite = afterWrite_In_rango_CH_2;
    if (canal == 3 )
    callback_rango . onWrite = afterWrite_In_rango_CH_3;
    if (canal == 4 )
    callback_rango . onWrite = afterWrite_In_rango_CH_4;
    if (canal == 5 )
    callback_rango . onWrite = afterWrite_In_rango_CH_5;
    if (canal == 6 )
    callback_rango . onWrite = afterWrite_In_rango_CH_6;
    if (canal == 7 )
    callback_rango . onWrite = afterWrite_In_rango_CH_7;

    UA_Server_setVariableNode_valueCallback ( server , in_rango_NodeId , callback_rango );
}
/****************************************** define: Salidas Analgicas ***************************************************/
static void
Define_object__salidas_Analogas(UA_Server *server, char* id_Analog_out) {

    UA_NodeId salidas_Analogas_Id = UA_NODEID_STRING(1, id_Analog_out ); /* get the nodeid assigned by the server */
    UA_ObjectAttributes salidas_Analogas_Attr = UA_ObjectAttributes_default;
    salidas_Analogas_Attr.displayName = UA_LOCALIZEDTEXT("es-ES", id_Analog_out);
    UA_Server_addObjectNode(server, salidas_Analogas_Id,UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),UA_QUALIFIEDNAME(1, id_Analog_out), UA_NODEID_NUMERIC(0, UA_NS0ID_BASEOBJECTTYPE),salidas_Analogas_Attr, NULL, &salidas_Analogas_Id);
}
static void
add_Out_out_Variable ( UA_Server * server,char * id_OUT ,char*id_Out_out ) {

    UA_UInt16 Out;
    UA_VariableAttributes Out_out_attr = UA_VariableAttributes_default ;
    Out_out_attr. displayName = UA_LOCALIZEDTEXT ( "es-ES" , "Out_out" );
    Out_out_attr. accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE ;
    Out_out_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    UA_Variant_setScalar ( &Out_out_attr . value , & Out , & UA_TYPES [ UA_TYPES_UINT16]);
    UA_NodeId Out_NodeId = UA_NODEID_STRING ( 1 , id_Out_out );
    UA_QualifiedName Out_out_Name = UA_QUALIFIEDNAME ( 1 , id_Out_out);
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Salidas_Analogas_Id = UA_NODEID_STRING(1, id_OUT );
    UA_Server_addVariableNode ( server , Out_NodeId , Salidas_Analogas_Id ,parentReferenceNodeId , Out_out_Name ,variableTypeNodeId , Out_out_attr , NULL ,NULL );
}
static void
add_Out_volts_Variable (UA_Server *server, char * id_OUT ,char* id_Out_volts){

    UA_Float Out_Volts;
    UA_VariableAttributes Out_Volts_attr = UA_VariableAttributes_default;
    Out_Volts_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Out_volts");
    Out_Volts_attr.dataType = UA_TYPES[UA_TYPES_FLOAT].typeId;
    Out_Volts_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    Out_Volts_attr.valueRank = -1;
    UA_Variant_setScalar(&Out_Volts_attr.value, &Out_Volts, &UA_TYPES[UA_TYPES_FLOAT]);
    UA_NodeId Out_Volts_nodeId = UA_NODEID_STRING ( 1 , id_Out_volts);
    UA_QualifiedName Out_volts_Name = UA_QUALIFIEDNAME ( 1 , id_Out_volts);
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Salidas_Analogas_Id = UA_NODEID_STRING(1, id_OUT );
    UA_Server_addVariableNode(server, Out_Volts_nodeId ,  Salidas_Analogas_Id,parentReferenceNodeId,Out_volts_Name,variableTypeNodeId, Out_Volts_attr, NULL, &Out_Volts_nodeId);
}

/**************************************************************************************/
void write_Out_volts(UA_Server *server,char *id_node_volts,int canal)
{
    float volts;
    volts = get_Out_Volts(canal);
    UA_Variant Out_volts;
    UA_Variant_setScalar(&Out_volts, &volts, &UA_TYPES[UA_TYPES_FLOAT]);
    UA_NodeId Out_nodeId_volts = UA_NODEID_STRING ( 1 ,id_node_volts );
    UA_Server_writeValue ( server , Out_nodeId_volts , Out_volts );
}
void write_Out_out(UA_Server *server,char *id_node_out,int canal)
{
     int out;
     out = get_Out(canal);
     UA_Variant Out_out_value;
     UA_Variant_setScalar(&Out_out_value, &out, &UA_TYPES[UA_TYPES_UINT16]);
     UA_NodeId Out_nodeId = UA_NODEID_STRING ( 1 ,id_node_out);
     UA_Server_writeValue ( server , Out_nodeId , Out_out_value );
}
void write_Out_porcent(UA_Server *server,char *id_node_porcent,int canal)
{
    float porcent;
    porcent = get_Out_porcent(canal);
    UA_Variant Out_porcent;
    UA_Variant_setScalar(&Out_porcent, &porcent, &UA_TYPES[UA_TYPES_FLOAT]);
    UA_NodeId Out_nodeId_porcent = UA_NODEID_STRING ( 1 ,id_node_porcent );
    UA_Server_writeValue ( server , Out_nodeId_porcent , Out_porcent );
}
/**************************************************************************************/
static void
add_Out_porcent_Variable (UA_Server *server, char * id_OUT ,char* id_Out_porcent){

    UA_Float Out_porcent;
    UA_VariableAttributes Out_porcent_attr = UA_VariableAttributes_default;
    Out_porcent_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Out_porcent");
    Out_porcent_attr.dataType = UA_TYPES[UA_TYPES_FLOAT].typeId;
    Out_porcent_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    Out_porcent_attr.valueRank = -1;
    UA_Variant_setScalar(&Out_porcent_attr.value, &Out_porcent, &UA_TYPES[UA_TYPES_FLOAT]);
    UA_NodeId Out_porcent_nodeId = UA_NODEID_STRING ( 1 , id_Out_porcent);
    UA_QualifiedName Out_porcent_Name = UA_QUALIFIEDNAME ( 1 , id_Out_porcent);
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Salidas_Analogas_Id = UA_NODEID_STRING(1, id_OUT );
    UA_Server_addVariableNode(server, Out_porcent_nodeId ,  Salidas_Analogas_Id,parentReferenceNodeId,Out_porcent_Name,variableTypeNodeId, Out_porcent_attr, NULL, &Out_porcent_nodeId);
}
///********************************** Salidas Analgicas - Out porcent: callback  ***************************************************/
static void
afterWrite_Out_porcent_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant Out_porcent_value0;
    Out_porcent_value0 = data -> value;
    void *v_r = Out_porcent_value0.data;
    float Out_porcent = *(UA_Float *)v_r;
    bool confir;
    confir=set_Out_porcent(0, Out_porcent);
}
 static void
afterWrite_Out_porcent_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant Out_porcent_value0;
    Out_porcent_value0 = data -> value;
    void *v_r = Out_porcent_value0.data;
    float Out_porcent = *(UA_Float *)v_r;
    bool confir;
    confir=set_Out_porcent(1, Out_porcent);
}
  static void
beforeRead_Out_porcent_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
     write_Out_volts(server,"OUT volts CH 0",0);
     write_Out_out(server,"OUT out CH 0",0);
}
 static void
beforeRead_Out_porcent_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    write_Out_volts(server,"OUT volts CH 1",1);
     write_Out_out(server,"OUT out CH 1",1);
}
static void
addValueCallbackTo_Out_porcent(UA_Server *server,char *id_nodo, int canal)
{
    UA_NodeId Out_porcent_NodeId = UA_NODEID_STRING ( 1 , id_nodo);
    UA_ValueCallback callback_Out_porcent ;
    if(canal == 0)
    {callback_Out_porcent . onRead =   beforeRead_Out_porcent_CH_0;
    callback_Out_porcent . onWrite = afterWrite_Out_porcent_CH_0;}
    if(canal == 1)
    {callback_Out_porcent . onRead =   beforeRead_Out_porcent_CH_0;
    callback_Out_porcent . onWrite = afterWrite_Out_porcent_CH_1;}
    UA_Server_setVariableNode_valueCallback ( server , Out_porcent_NodeId , callback_Out_porcent);
}

///********************************** Salidas Analgicas - Out out: callback  ***************************************************/
  static void
beforeRead_Out_out_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
     write_Out_volts(server,"OUT volts CH 0",0);
     write_Out_porcent(server,"OUT porcent CH 0",0);
}
 static void
beforeRead_Out_out_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    write_Out_volts(server,"OUT volts CH 1",1);
     write_Out_porcent(server,"OUT porcent CH 1",1);
}
 static void
afterWrite_Out_out_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant Out_out_value0;
    Out_out_value0 = data -> value;
    void *v_r = Out_out_value0.data;
    int Out_out = *(UA_UInt16 *)v_r;
    bool confir;
    confir=set_Out(0, Out_out);
}
 static void
afterWrite_Out_out_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant Out_out_value1;
    Out_out_value1 = data -> value;
    void *v_r = Out_out_value1.data;
    int Out_out = *(UA_UInt16 *)v_r;
    bool confir;
    confir=set_Out(1, Out_out);
}
static void
addValueCallbackTo_Out_out(UA_Server *server,char *id_nodo, int canal)
{
    UA_NodeId Out_out_NodeId = UA_NODEID_STRING ( 1 , id_nodo);
    UA_ValueCallback callback_Out_out ;
    if(canal == 0)
    {callback_Out_out . onRead =   beforeRead_Out_out_CH_0;
    callback_Out_out . onWrite = afterWrite_Out_out_CH_0;}
    if(canal == 1)
    {callback_Out_out . onRead =   beforeRead_Out_out_CH_0;
    callback_Out_out . onWrite = afterWrite_Out_out_CH_1;}
    UA_Server_setVariableNode_valueCallback ( server , Out_out_NodeId , callback_Out_out );
}
///********************************** Salidas Analgicas - Out volts: callback  ***************************************************/
static void
beforeRead_Out_volts_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
     write_Out_out(server,"OUT out CH 0",0);
      write_Out_porcent(server,"OUT porcent CH 0",0);
}
static void
beforeRead_Out_volts_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    write_Out_out(server,"OUT out CH 1",1);
     write_Out_porcent(server,"OUT porcent CH 1",1);
}
static void
afterWrite_Out_volts ( UA_Server * server,char *id_node,char *id_node_out,int canal) {

    UA_Variant Out_volts_value;
    UA_NodeId Out_nodeId_volts = UA_NODEID_STRING ( 1 ,id_node );
    UA_Server_readValue ( server , Out_nodeId_volts , &Out_volts_value );
    void *v_r = Out_volts_value.data;
    float Out_volts = *(UA_Float *)v_r;
    bool confir=false;
    confir=set_Out_Volts(canal, Out_volts);
}
 static void
afterWrite_Out_volts_CH_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_Out_volts ( server,"OUT volts CH 0","OUT out CH 0"  ,0);
}
 static void
afterWrite_Out_volts_CH_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_Out_volts ( server,"OUT volts CH 1","OUT out CH 1"  ,1);
}
static void
addValueCallbackTo_Out_volts(UA_Server *server,char *id_nodo, int canal)
{
    UA_NodeId Out_volts_NodeId = UA_NODEID_STRING ( 1 , id_nodo);
    UA_ValueCallback callback_Out_volts ;
    if(canal == 0)
    {callback_Out_volts.onRead = beforeRead_Out_volts_CH_0;
    callback_Out_volts. onWrite = afterWrite_Out_volts_CH_0;}
    if(canal == 1)
    {callback_Out_volts.onRead = beforeRead_Out_volts_CH_1;
    callback_Out_volts . onWrite = afterWrite_Out_volts_CH_1;}

    UA_Server_setVariableNode_valueCallback ( server , Out_volts_NodeId , callback_Out_volts );
}
/********************************************************************/
void Diferencial_mode(UA_Server *server)
{
                     /** Entradas Analgicas **/
          Define_object_entradas_Analogas(server,"Analog CH IN 0");
          Define_object_entradas_Analogas(server,"Analog CH IN 1");
          Define_object_entradas_Analogas(server,"Analog CH IN 2");
          Define_object_entradas_Analogas(server,"Analog CH IN 3");

          add_In_Volts_Variable ( server,"Analog CH IN 0","In volts CH 0");
          add_In_Volts_Variable ( server,"Analog CH IN 1","In volts CH 1");
          add_In_Volts_Variable ( server,"Analog CH IN 2","In volts CH 2");
          add_In_Volts_Variable ( server,"Analog CH IN 3","In volts CH 3");

          addValueCallbackTo_in_volts(server,"In volts CH 0",0);
          addValueCallbackTo_in_volts(server,"In volts CH 1",1);
          addValueCallbackTo_in_volts(server,"In volts CH 2",2);
          addValueCallbackTo_in_volts(server,"In volts CH 3",3);

          add_In_Bin_Variable ( server,"Analog CH IN 0","In bin CH 0");
          add_In_Bin_Variable ( server,"Analog CH IN 1","In bin CH 1");
          add_In_Bin_Variable ( server,"Analog CH IN 2","In bin CH 2");
          add_In_Bin_Variable ( server,"Analog CH IN 3","In bin CH 3");

          addValueCallbackTo_in_bin(server,"In bin CH 0",0);
          addValueCallbackTo_in_bin(server,"In bin CH 1",1);
          addValueCallbackTo_in_bin(server,"In bin CH 2",2);
          addValueCallbackTo_in_bin(server,"In bin CH 3",3);

         add_In_Rango_Variable ( server,"Analog CH IN 0" ,"In rango CH 0" );
         add_In_Rango_Variable ( server,"Analog CH IN 1" ,"In rango CH 1" );
         add_In_Rango_Variable ( server,"Analog CH IN 2" ,"In rango CH 2" );
         add_In_Rango_Variable ( server,"Analog CH IN 3" ,"In rango CH 3" );

         addValueCallbackTo_in_rango(server,"In rango CH 0",0);
         addValueCallbackTo_in_rango(server,"In rango CH 1",1);
         addValueCallbackTo_in_rango(server,"In rango CH 2",2);
         addValueCallbackTo_in_rango(server,"In rango CH 3",3);

                    /** Salidas Analgicas **/
         Define_object__salidas_Analogas(server, "Analog CH OUT 0");
         Define_object__salidas_Analogas(server, "Analog CH OUT 1");

         add_Out_out_Variable ( server,"Analog CH OUT 0" ,"OUT out CH 0");
         addValueCallbackTo_Out_out(server,"OUT out CH 0",0);

         add_Out_out_Variable ( server,"Analog CH OUT 1" ,"OUT out CH 1");
         addValueCallbackTo_Out_out(server,"OUT out CH 1",1);

         add_Out_volts_Variable( server,"Analog CH OUT 0" ,"OUT volts CH 0");
         add_Out_volts_Variable ( server,"Analog CH OUT 1" ,"OUT volts CH 1");

        addValueCallbackTo_Out_volts(server,"OUT volts CH 0",0);
        addValueCallbackTo_Out_volts(server,"OUT volts CH 1",1);

        add_Out_porcent_Variable (server,"Analog CH OUT 0","OUT porcent CH 0");
        add_Out_porcent_Variable (server,"Analog CH OUT 1","OUT porcent CH 1");

        addValueCallbackTo_Out_porcent(server,"OUT porcent CH 0",0);
        addValueCallbackTo_Out_porcent(server,"OUT porcent CH 1",1);
}
/********************************************************************/
void Comun_mode(UA_Server *server)
{

      Define_object_entradas_Analogas(server,"Analog CH IN 4");
      Define_object_entradas_Analogas(server,"Analog CH IN 5");
      Define_object_entradas_Analogas(server,"Analog CH IN 6");
      Define_object_entradas_Analogas(server,"Analog CH IN 7");

      add_In_Volts_Variable ( server,"Analog CH IN 4","In volts CH 4");
      add_In_Volts_Variable ( server,"Analog CH IN 5","In volts CH 5");
      add_In_Volts_Variable ( server,"Analog CH IN 6","In volts CH 6");
      add_In_Volts_Variable ( server,"Analog CH IN 7","In volts CH 7");

      addValueCallbackTo_in_volts(server,"In volts CH 4",4);
      addValueCallbackTo_in_volts(server,"In volts CH 5",5);
      addValueCallbackTo_in_volts(server,"In volts CH 6",6);
      addValueCallbackTo_in_volts(server,"In volts CH 7",7);

      add_In_Bin_Variable ( server,"Analog CH IN 4","In bin CH 4");
      add_In_Bin_Variable ( server,"Analog CH IN 5","In bin CH 5");
      add_In_Bin_Variable ( server,"Analog CH IN 6","In bin CH 6");
      add_In_Bin_Variable ( server,"Analog CH IN 7","In bin CH 7");

      addValueCallbackTo_in_bin(server,"In bin CH 4",4);
      addValueCallbackTo_in_bin(server,"In bin CH 5",5);
      addValueCallbackTo_in_bin(server,"In bin CH 6",6);
      addValueCallbackTo_in_bin(server,"In bin CH 7",7);

      add_In_Rango_Variable ( server,"Analog CH IN 4" ,"In rango CH 4" );
      add_In_Rango_Variable ( server,"Analog CH IN 5" ,"In rango CH 5" );
      add_In_Rango_Variable ( server,"Analog CH IN 6" ,"In rango CH 6" );
      add_In_Rango_Variable ( server,"Analog CH IN 7" ,"In rango CH 7" );

      addValueCallbackTo_in_rango(server,"In rango CH 4",4);
      addValueCallbackTo_in_rango(server,"In rango CH 5",5);
      addValueCallbackTo_in_rango(server,"In rango CH 6",6);
      addValueCallbackTo_in_rango(server,"In rango CH 7",7);
}

/********************************************************************************************************************/
/********************************** Define: Ports Digitales  ******************************************************/
/********************************************************************************************************************/

/******************************************* BIT dig ************************************************/
 static void
add_dig_bit(UA_Server *server,char *Port_a,char * dig_bit_id){

    UA_VariableAttributes dig_bit_attr = UA_VariableAttributes_default;
    UA_UInt16 bit_dig_id_num = 0;
    UA_Variant_setScalar(&dig_bit_attr.value, &bit_dig_id_num, &UA_TYPES[UA_TYPES_UINT16]);
    dig_bit_attr.description = UA_LOCALIZEDTEXT("es-ES",dig_bit_id);
    dig_bit_attr.displayName = UA_LOCALIZEDTEXT("es-ES",dig_bit_id);
    dig_bit_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    dig_bit_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    UA_NodeId dig_bit_Id= UA_NODEID_STRING ( 1 ,dig_bit_id);
    UA_QualifiedName dig_bit_Name = UA_QUALIFIEDNAME ( 1 ,dig_bit_id);
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Port_a_Id = UA_NODEID_STRING(1, Port_a );
    UA_Server_addVariableNode ( server , dig_bit_Id , Port_a_Id ,parentReferenceNodeId , dig_bit_Name ,variableTypeNodeId , dig_bit_attr , NULL , NULL );
}
/******************************************* Port DIGITAL A  ****************************************/
static void
Define_Port_dig_a (UA_Server *server, char* id_Port) {

    UA_NodeId Port_a_Id = UA_NODEID_STRING(1,id_Port);
    UA_ObjectAttributes p_a__Attr = UA_ObjectAttributes_default;
    p_a__Attr.displayName = UA_LOCALIZEDTEXT("es-ES", id_Port);
    UA_Server_addObjectNode(server, Port_a_Id,UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),UA_QUALIFIEDNAME(1, id_Port), UA_NODEID_NUMERIC(0, UA_NS0ID_BASEOBJECTTYPE),p_a__Attr, NULL, &Port_a_Id);
}
/******************************************* Port A config ****************************************/
static void
add_Port_a_config(UA_Server *server,char *Port_a){

    UA_VariableAttributes config_a_attr = UA_VariableAttributes_default;
    UA_UInt16 config_a_id_num = 0;
    UA_Variant_setScalar(&config_a_attr.value, &config_a_id_num, &UA_TYPES[UA_TYPES_UINT16]);
    config_a_attr.description = UA_LOCALIZEDTEXT("es-ES","Modo: In/0 Out/1");
    config_a_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Modo: In/0 Out/1");
    config_a_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    config_a_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    UA_NodeId config_a_Id= UA_NODEID_STRING ( 1 ,"Port a");
    UA_QualifiedName config_a_Name = UA_QUALIFIEDNAME ( 1 , "Port a");
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Port_a_Id = UA_NODEID_STRING(1, Port_a );
    UA_Server_addVariableNode ( server , config_a_Id , Port_a_Id ,parentReferenceNodeId , config_a_Name ,variableTypeNodeId , config_a_attr , NULL , NULL );
}
///********************************** Port A configuracin : callback **************************************/
 static void
afterWrite_config_a ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant config_a_value;
    UA_NodeId config_a_nodeId = UA_NODEID_STRING ( 1 ,"Port a" );
    UA_Server_readValue ( server , config_a_nodeId , &config_a_value );
    void *v_r = config_a_value.data;
    int config_a_modo = *(UA_UInt16 *)v_r;
    // 0 In / 1 Out
    if (config_a_modo == 0)
    {
        set_digmode_AIN();
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital A : Modo entrada ---- [%d]\n\n ",config_a_modo);
    }
    if (config_a_modo ==1)
    {
        set_digmode_AOUT();
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital A : Modo Salida ---- [%d]\n\n ",config_a_modo);
    }
     if (config_a_modo !=1 && config_a_modo !=0)
         UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital A : Modo INCORRECTO ---- \n\n ");
}
static void
addValueCallbackTo_config_a_Variable ( UA_Server * server) {

    UA_NodeId config_a_nodeId = UA_NODEID_STRING ( 1 , "Port a");
    UA_ValueCallback callback_config_a ;
    callback_config_a . onWrite = afterWrite_config_a;
    UA_Server_setVariableNode_valueCallback ( server , config_a_nodeId , callback_config_a );
}
/************************ before read bit A ************************/
void beforeRead_dig_bit_a(UA_Server *server,char * id_node_bit,int canal)
{
    int modo;
    modo = get_digmode_a();
    UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ : modo = [%d] \n\n ",modo);
    if (modo == 0) // Modo entrada
    {
    UA_NodeId dig_bit_nodeId = UA_NODEID_STRING ( 1 , id_node_bit);
    UA_UInt16 dig_bit = get_bit_Di_A(canal);
    UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ beforeRead bit A 1 : valor del bit = [%d] \n\n ",dig_bit);
    UA_Variant dig_bit_value ;
    UA_Variant_setScalar ( &dig_bit_value , & dig_bit , & UA_TYPES [ UA_TYPES_UINT16]);
    UA_Server_writeValue ( server , dig_bit_nodeId , dig_bit_value );
    }
}
/************************ after write bit A ************************/
void afterWrite_dig_bit_a(UA_Server *server,char * id_node_bit,int canal)
{
    int modo;
    modo = get_digmode_a();
    UA_Variant dig_bit_value;
    unsigned int dig_bit = 0;
    UA_NodeId dig_bit_nodeId = UA_NODEID_STRING ( 1 ,id_node_bit );
    if (modo == 1)
    {
    UA_Server_readValue ( server , dig_bit_nodeId , &dig_bit_value );
    void *v_r = dig_bit_value.data;
    int bit = *(UA_UInt16 *)v_r;
    if (bit ==0 || bit ==1)
    {
        bool confir;
        confir = set_bit_Do_A(canal,bit);
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------afterWrite BIT ---- [%d]\n\n ",bit);
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------afterWrite canal ---- [%d]\n\n ",canal);
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ CONFIRMACIN ---- [%d]\n\n ",confir);
    }
        if (bit !=0 && bit !=1)
        {
        UA_Variant_setScalar(&dig_bit_value,&dig_bit,&UA_TYPES[UA_TYPES_UINT16]);
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ valor [%d] INCORRECTO : NO ENVIADO A 1408FS---- \n\n ",bit);
        UA_Server_writeValue(server,dig_bit_nodeId,dig_bit_value);
        }
    }
}
 static void
beforeRead_dig_bit_A_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 0",0);
}
 static void
afterWrite_dig_bit_A_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 0",0);
}
 static void
beforeRead_dig_bit_A_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 1",1);
}
 static void
afterWrite_dig_bit_A_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 1",1);
}
 static void
beforeRead_dig_bit_A_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 2",2);
}
 static void
afterWrite_dig_bit_A_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 2",2);
}
 static void
beforeRead_dig_bit_A_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 3",3);
}
 static void
afterWrite_dig_bit_A_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 3",3);
}
 static void
beforeRead_dig_bit_A_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 4",4);
}
 static void
afterWrite_dig_bit_A_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 4",4);
}
 static void
beforeRead_dig_bit_A_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 5",5);
}
 static void
afterWrite_dig_bit_A_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 5",5);
}
 static void
beforeRead_dig_bit_A_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 6",6);
}
 static void
afterWrite_dig_bit_A_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 6",6);
}
 static void
beforeRead_dig_bit_A_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_a(server,"A bit 7",7);
}
 static void
afterWrite_dig_bit_A_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_a(server,"A bit 7",7);
}
static void
addValueCallbackTo_dig_bit_A ( UA_Server * server,char * id_node_bit_a,int canal) {

    UA_NodeId dig_bit_nodeId = UA_NODEID_STRING ( 1 , id_node_bit_a);
    UA_ValueCallback callback_dig_bit ;

            if(canal == 0)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_0;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_0;}
            if(canal == 1)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_1;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_1;}
            if(canal == 2)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_2;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_2;}
            if(canal == 3)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_3;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_3;}
            if(canal == 4)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_4;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_4;}
            if(canal == 5)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_5;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_5;}
            if(canal == 6)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_6;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_6;}
            if(canal == 7)
        {callback_dig_bit.onWrite = afterWrite_dig_bit_A_7;
        callback_dig_bit.onRead = beforeRead_dig_bit_A_7;}

    UA_Server_setVariableNode_valueCallback ( server , dig_bit_nodeId , callback_dig_bit );
}
/******************************************* Port DIGITAL B  ****************************************/
static void
Define_Port_dig_b (UA_Server *server, char* id_Port) {

    UA_NodeId Port_a_Id = UA_NODEID_STRING(1,id_Port);
    UA_ObjectAttributes p_a__Attr = UA_ObjectAttributes_default;
    p_a__Attr.displayName = UA_LOCALIZEDTEXT("es-ES", id_Port);
    UA_Server_addObjectNode(server, Port_a_Id,UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER),UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),UA_QUALIFIEDNAME(1, id_Port), UA_NODEID_NUMERIC(0, UA_NS0ID_BASEOBJECTTYPE),p_a__Attr, NULL, &Port_a_Id);
}
///********************************* Port B configuracin **************************************/
static void
add_Port_b_config(UA_Server *server,char *Port_a){

    UA_VariableAttributes config_b_attr = UA_VariableAttributes_default;
    UA_UInt16 config_b_id_num = 0;
    UA_Variant_setScalar(&config_b_attr.value, &config_b_id_num, &UA_TYPES[UA_TYPES_UINT16]);
    config_b_attr.description = UA_LOCALIZEDTEXT("es-ES","Modo: In/0  Out/1");
    config_b_attr.displayName = UA_LOCALIZEDTEXT("es-ES","Modo: In/0  Out/1");
    config_b_attr.dataType = UA_TYPES[UA_TYPES_UINT16].typeId;
    config_b_attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
    config_b_attr.valueRank = -1;
    UA_NodeId config_b_Id= UA_NODEID_STRING ( 1 ,"Port b");
    UA_QualifiedName config_b_Name = UA_QUALIFIEDNAME ( 1 , "Port b");
    UA_NodeId parentReferenceNodeId =  UA_NODEID_NUMERIC(0, UA_NS0ID_HASCOMPONENT);
    UA_NodeId variableTypeNodeId = UA_NODEID_NUMERIC ( 0 , UA_NS0ID_BASEDATAVARIABLETYPE );
    UA_NodeId Port_b_Id = UA_NODEID_STRING(1, Port_a );
    UA_Server_addVariableNode ( server , config_b_Id , Port_b_Id ,parentReferenceNodeId , config_b_Name ,variableTypeNodeId , config_b_attr , NULL , NULL );
}
///********************************** Port B config_modo callback **************************************/
 static void
afterWrite_config_b ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {

    UA_Variant config_b_value;
    UA_NodeId config_b_nodeId = UA_NODEID_STRING ( 1 ,"Port b" );
    UA_Server_readValue ( server , config_b_nodeId , &config_b_value );
    void *v_r = config_b_value.data;
    int config_b_modo = *(UA_UInt16 *)v_r;
    UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------afterWrite_config_modo B ---- [%d]\n\n ",config_b_modo);
    if (config_b_modo == 0)
    {
        set_digmode_BIN();
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital B : Modo entrada ---- [%d]\n\n ",config_b_modo);
    }
    if (config_b_modo ==1)
    {
        set_digmode_BOUT();
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital B : Modo Salida ---- [%d]\n\n ",config_b_modo);
    }
     if (config_b_modo !=1 && config_b_modo !=0)
     {
         UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n\n------ Port digital B : Modo INCORRECTO ---- \n\n ");
     }
}
static void
addValueCallbackTo_config_b_Variable ( UA_Server * server) {

    UA_NodeId config_b_nodeId = UA_NODEID_STRING ( 1 , "Port b");
    UA_ValueCallback callback_config_b ;
    callback_config_b . onWrite = afterWrite_config_b;
    UA_Server_setVariableNode_valueCallback ( server , config_b_nodeId , callback_config_b );
}
/************************ before read bit A ************************/
void beforeRead_dig_bit_b(UA_Server *server,char * id_node_bit,int canal)
{
    int modo;
    modo = get_digmode_b();
    if (modo == 0) // Modo entrada
    {
    UA_NodeId dig_bit_nodeId = UA_NODEID_STRING ( 1 , id_node_bit);
    UA_UInt16 dig_bit = get_bit_Di_B(canal);
    UA_Variant dig_bit_value ;
    UA_Variant_setScalar ( &dig_bit_value , & dig_bit , & UA_TYPES [ UA_TYPES_UINT16]);
    UA_Server_writeValue ( server , dig_bit_nodeId , dig_bit_value );
    }
}
/************************ after write bit A ************************/
void afterWrite_dig_bit_b(UA_Server *server,char * id_node_bit,int canal)
{
    int modo;
    modo = get_digmode_b();
    UA_Variant dig_bit_value;
    unsigned int dig_bit = 0;
    UA_NodeId dig_bit_nodeId = UA_NODEID_STRING ( 1 ,id_node_bit );
    if (modo == 1)
    {
    UA_Server_readValue ( server , dig_bit_nodeId , &dig_bit_value );
    void *v_r = dig_bit_value.data;
    int bit = *(UA_UInt16 *)v_r;
    if (bit ==0 || bit ==1)
    {
        set_bit_Do_B(canal,bit);
    }
        if (bit !=0 && bit !=1)
        {
        UA_Variant_setScalar(&dig_bit_value,&dig_bit,&UA_TYPES[UA_TYPES_UINT16]);
        UA_Server_writeValue(server,dig_bit_nodeId,dig_bit_value);
        }
    }
}
/************************** BIT dig B : callback ********************************/
 static void
beforeRead_dig_bit_B_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 0",0);
}
 static void
afterWrite_dig_bit_B_0 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 0",0);
}
 static void
beforeRead_dig_bit_B_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 1",1);
}
 static void
afterWrite_dig_bit_B_1 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 1",1);
}
 static void
beforeRead_dig_bit_B_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 2",2);
}
 static void
afterWrite_dig_bit_B_2 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 2",2);
}
 static void
beforeRead_dig_bit_B_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 3",3);
}
 static void
afterWrite_dig_bit_B_3 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 3",3);
}
 static void
beforeRead_dig_bit_B_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 4",4);
}
 static void
afterWrite_dig_bit_B_4 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 4",4);
}
 static void
beforeRead_dig_bit_B_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 5",5);
}
 static void
afterWrite_dig_bit_B_5 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 5",5);
}
 static void
beforeRead_dig_bit_B_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 6",6);
}
 static void
afterWrite_dig_bit_B_6 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 6",6);
}
 static void
beforeRead_dig_bit_B_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    beforeRead_dig_bit_b(server,"B bit 7",7);
}
 static void
afterWrite_dig_bit_B_7 ( UA_Server * server ,const UA_NodeId * sessionId , void * sessionContext ,const UA_NodeId * nodeid , void * nodeContext ,const UA_NumericRange * range , const UA_DataValue * data) {
    afterWrite_dig_bit_b(server,"B bit 7",7);
}
static void
addValueCallbackTo_dig_bit_B ( UA_Server * server,char * id_node_bit_b, int canal) {

    UA_NodeId dig_bit_nodeId_b = UA_NODEID_STRING ( 1 ,id_node_bit_b);
    UA_ValueCallback callback_dig_bit_b ;

        if(canal == 0)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_0;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_0;}
        if(canal == 1)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_1;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_1;}
        if(canal == 2)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_2;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_2;}
        if(canal == 3)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_3;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_3;}
        if(canal == 4)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_4;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_4;}
        if(canal == 5)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_5;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_5;}
        if(canal == 6)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_6;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_6;}
        if(canal == 7)
    {callback_dig_bit_b.onWrite = afterWrite_dig_bit_B_7;
    callback_dig_bit_b.onRead = beforeRead_dig_bit_B_7;}

    UA_Server_setVariableNode_valueCallback ( server , dig_bit_nodeId_b , callback_dig_bit_b );
}
/******************************** Add Ports digitales **********************************/
static
void add_Ports_digitales(UA_Server *server)
{
    Define_Port_dig_a(server,"Port Digital A");
    add_Port_a_config(server,"Port Digital A");
    addValueCallbackTo_config_a_Variable(server);

    add_dig_bit (server,"Port Digital A","A bit 0");
    add_dig_bit (server,"Port Digital A","A bit 1");
    add_dig_bit (server,"Port Digital A","A bit 2");
    add_dig_bit (server,"Port Digital A","A bit 3");
    add_dig_bit (server,"Port Digital A","A bit 4");
    add_dig_bit (server,"Port Digital A","A bit 5");
    add_dig_bit (server,"Port Digital A","A bit 6");
    add_dig_bit (server,"Port Digital A","A bit 7");
//
    addValueCallbackTo_dig_bit_A(server,"A bit 0",0);
    addValueCallbackTo_dig_bit_A(server,"A bit 1",1);
    addValueCallbackTo_dig_bit_A(server,"A bit 2",2);
    addValueCallbackTo_dig_bit_A(server,"A bit 3",3);
    addValueCallbackTo_dig_bit_A(server,"A bit 4",4);
    addValueCallbackTo_dig_bit_A(server,"A bit 5",5);
    addValueCallbackTo_dig_bit_A(server,"A bit 6",6);
    addValueCallbackTo_dig_bit_A(server,"A bit 7",7);
//
    Define_Port_dig_b(server,"Port Digital B");
    add_Port_b_config(server,"Port Digital B");
    addValueCallbackTo_config_b_Variable(server);
//
    add_dig_bit (server,"Port Digital B","B bit 0");
    add_dig_bit (server,"Port Digital B","B bit 1");
    add_dig_bit (server,"Port Digital B","B bit 2");
    add_dig_bit (server,"Port Digital B","B bit 3");
    add_dig_bit (server,"Port Digital B","B bit 4");
    add_dig_bit (server,"Port Digital B","B bit 5");
    add_dig_bit (server,"Port Digital B","B bit 6");
    add_dig_bit (server,"Port Digital B","B bit 7");

    addValueCallbackTo_dig_bit_B(server,"B bit 0",0);
    addValueCallbackTo_dig_bit_B(server,"B bit 1",1);
    addValueCallbackTo_dig_bit_B(server,"B bit 2",2);
    addValueCallbackTo_dig_bit_B(server,"B bit 3",3);
    addValueCallbackTo_dig_bit_B(server,"B bit 4",4);
    addValueCallbackTo_dig_bit_B(server,"B bit 5",5);
    addValueCallbackTo_dig_bit_B(server,"B bit 6",6);
    addValueCallbackTo_dig_bit_B(server,"B bit 7",7);

}
static
void add_tarjeta(UA_Server *server)
{
    define_tarjeta(server,"1408FS-PLUS");
    add_config_modo(server,"1408FS-PLUS");
}
/***************************************************** SERVER OPC UA  *************************************************************************/
//UA_Boolean running = true;
static void stopHandler(int sign) {
    UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "received ctrl-c");
    running = false;
}

int main ( void ) {
    signal ( SIGINT , stopHandler );
    signal ( SIGTERM , stopHandler );

    UA_ServerConfig *config =  UA_ServerConfig_new_minimal(1408,NULL);
    UA_Server * server = UA_Server_new ( config );

    ini_tarjeta();
    init_entradas();
    init_salidas();
    init_digitales();
    /* tarjeta 1408FS */
    add_tarjeta(server);
    /* Cofiguracin: cbGetConfig*/// diferencial / comn
    Diferencial_mode(server);
    if (read_config_modo(server) == 4)
        UA_LOG_INFO(UA_Log_Stdout, UA_LOGCATEGORY_SERVER, "\n \n Modo diferencial");
    if (read_config_modo(server) == 8)
    Comun_mode(server);
    /* Ports digitales */
    add_Ports_digitales(server);

    UA_StatusCode retval = UA_Server_run ( server , & running );
    UA_Server_delete (server);
    UA_ServerConfig_delete ( config );
    return ( int ) retval ;
}
